package com.danitheskunk.skunkworks;

import org.lwjgl.glfw.GLFWErrorCallback;
import org.lwjgl.opengl.GL;
import org.lwjgl.opengl.GL11;
import org.lwjgl.stb.STBRectPack;

import static org.lwjgl.glfw.GLFW.*;
import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.system.MemoryUtil.NULL;

public class GLWindow implements IWindow {
	private Engine engine;
	private GLRenderContext renderContext;
	private boolean shouldClose;
	private GLTextureAtlas textureAtlas;
	private long window;

	public GLWindow(int width, int height, String title, Engine engine) {
		GLFWErrorCallback.createPrint(System.err).set();
		if(!glfwInit()) throw new IllegalStateException("Unable to initialize GLFW");

		glfwDefaultWindowHints();

		this.engine = engine;

		window = glfwCreateWindow(width, height, title, NULL, NULL);
		if(window == NULL) throw new RuntimeException("Failed to create GLFW window");


		glfwShowWindow(window);
		glfwMakeContextCurrent(window);
		glfwSwapInterval(1); //vsync
		GL.createCapabilities();
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0.0f, width, height, 0.0f, 0.0f, 1.0f);
		glEnable(GL_COLOR);
		glEnable(GL_TEXTURE_2D);

		renderContext = new GLRenderContext();
		textureAtlas = new GLTextureAtlas();

		shouldClose = false;

		System.out.println("Skunkworks window initialised");
		System.out.print("Maximum Texture Size: ");
		System.out.println(GL11.glGetInteger(GL_MAX_TEXTURE_SIZE));
	}

	@Override
	public ITexture loadTexture(String path) {
		var img = engine.loadImage(path);
		return textureAtlas.addTexture(img);
	}

	@Override
	public void renderFinish(IRenderContext context) {
		glEnd();
		glfwSwapBuffers(window);
	}

	@Override
	public IRenderContext renderStart() {
		glClearColor(0.1f, 0.2f, 0.3f, 1.0f);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glBegin(GL_TRIANGLES);
		return renderContext;
	}

	@Override
	public boolean shouldClose() {
		return shouldClose;
	}

	@Override
	public void tick() {
		glfwMakeContextCurrent(window);
		if(glfwWindowShouldClose(window)) {
			shouldClose = true;
			return;
		}

		glfwPollEvents();
	}
}
